package com.itblogs.config;

import java.util.Properties;

import javax.sql.DataSource;
 
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.env.Environment;
import org.apache.commons.dbcp2.BasicDataSource;
import org.springframework.orm.hibernate4.HibernateTransactionManager;
import org.springframework.orm.hibernate4.LocalSessionFactoryBean;
import org.springframework.transaction.annotation.EnableTransactionManagement;

/**
 * @author      Kelvin Hui
 * @see       	it-blogs.com
 * @since     	01-Jan-2016
 */

@Configuration
@ComponentScan({ "com.itblogs.config" })
@EnableTransactionManagement
@PropertySource(value = { "classpath:application.properties" })
public class HibernateConfiguration {

    private static final String CONNECTION_DRIVERCLASSNAME 		= "connection.driverClassName";
    private static final String CONNECTION_URL 					= "connection.url";
    private static final String CONNECTION_USERNAME 			= "connection.username";
    private static final String CONNECTION_PASSWORD 			= "connection.password";
    private static final String DBCP_INITIALSIZE	 			= "dbcp.initialSize";
    private static final String DBCP_MAXTOTAL 					= "dbcp.maxTotal";
    private static final String DBCP_MAXIDLE				 	= "dbcp.maxIdle";
    private static final String HIBERNATE_DIALECT 				= "hibernate.dialect";
    private static final String HIBERNATE_SHOW_SQL 				= "hibernate.show_sql";
    private static final String HIBERNATE_FORMAT_SQL 			= "hibernate.format_sql";
    private static final String PACKAGES_TO_SCAN 				= "com.itblogs.model";	
	
    private static final int DBCP_INITIAL_SIZE		 			= 10;
    private static final int DBCP_MAX_TOTAL		 				= 50;
    private static final int DBCP_MAX_IDLE		 				= 10;
    
	@Autowired
    private Environment environment;
 
    @Bean
    public LocalSessionFactoryBean sessionFactory() {
        LocalSessionFactoryBean sessionFactory = new LocalSessionFactoryBean();
        sessionFactory.setDataSource(dataSource());
        sessionFactory.setPackagesToScan(new String[] { PACKAGES_TO_SCAN });
        
        Properties properties = new Properties();
        properties.put(HIBERNATE_DIALECT, environment.getRequiredProperty( HIBERNATE_DIALECT ));
        properties.put(HIBERNATE_SHOW_SQL, environment.getRequiredProperty( HIBERNATE_SHOW_SQL ));
        properties.put(HIBERNATE_FORMAT_SQL, environment.getRequiredProperty(HIBERNATE_FORMAT_SQL));
        sessionFactory.setHibernateProperties(properties);
        
        return sessionFactory;
     }
     
    @Bean
    public DataSource dataSource() {
    	BasicDataSource dataSource = new BasicDataSource();
        dataSource.setDriverClassName(environment.getRequiredProperty( CONNECTION_DRIVERCLASSNAME ));
        dataSource.setUrl(environment.getRequiredProperty( CONNECTION_URL ));
        dataSource.setUsername(environment.getRequiredProperty( CONNECTION_USERNAME ));
        dataSource.setPassword(environment.getRequiredProperty( CONNECTION_PASSWORD ));
        
        int initialSize = DBCP_INITIAL_SIZE;
        int maxTotal = DBCP_MAX_TOTAL;
        int maxIdle = DBCP_MAX_IDLE;
        
        try { initialSize = Integer.parseInt( environment.getRequiredProperty( DBCP_INITIALSIZE ) ); } catch(Exception ie) { }
        try { maxTotal = Integer.parseInt( environment.getRequiredProperty( DBCP_MAXTOTAL ) ); } catch(Exception ie) { }
        try { maxIdle = Integer.parseInt( environment.getRequiredProperty( DBCP_MAXIDLE ) ); } catch(Exception ie) { }
        dataSource.setInitialSize(initialSize);
        dataSource.setMaxTotal(maxTotal);
        dataSource.setMaxIdle(maxIdle);
        
        return dataSource;
    }
     
    @Bean
    @Autowired
    public HibernateTransactionManager transactionManager(SessionFactory sessionFactory) {
       HibernateTransactionManager txManager = new HibernateTransactionManager();
       txManager.setSessionFactory(sessionFactory);
       return txManager;
    }
	
}
