package com.itblogs.controller;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import javax.validation.Valid;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import com.itblogs.service.*;
import com.itblogs.model.*;

/**
 * @author      Kelvin Hui
 * @see       	it-blogs.com
 * @since     	01-Jan-2016
 */

@Controller
@RequestMapping("/customer")
public class CustomerController {

	private static final Logger logger = Logger.getLogger(CustomerController.class);
	
	@Autowired
	CustomerService customerService;
	
	@Autowired
	OrdersService ordersService;
	
    @Autowired
    MessageSource messageSource;

    
	/**
	 * Listing all existing records
	 */    
    @RequestMapping(value = { "/", "/list" }, method = RequestMethod.GET)
    public String list(ModelMap model, String msg) {
    	List<Customer> customers = customerService.list(null,null);
    	model.addAttribute("customers", customers);
    	model.addAttribute("msg", msg);
    	
        return "customer/list";
    }
 
    /**
	 * New form
	 */
	@RequestMapping(value = { "/new" }, method = RequestMethod.GET)
	public String newForm(ModelMap model) {
		Customer customer = new Customer();
		model.addAttribute("customer", customer);		
		model.addAttribute("isNew", true);		
		
		return "customer/form";
	}
 
	/**
	 * Save new
	 */
	@RequestMapping(value = { "/new" }, method = RequestMethod.POST)
	public String save(
			@Valid Customer customer, BindingResult result, 
			ModelMap model) {
		if (result.hasErrors()) {
			model.addAttribute("error", "Customer " + customer.getEmail() + " created failed");
			model.addAttribute("customer", customer);
			model.addAttribute("isNew", true);
			
			return "customer/form";
		}

		//Save Customer		
		customerService.save(customer);

		model.addAttribute("success", "Customer " + customer.getEmail() + " created successfully");
		model.addAttribute("customer", customer);
		model.addAttribute("isNew", false);
		
		return "customer/form";
	}
	
	/**
	 * Update form
	 */
	@RequestMapping(value = { "/update/{id}" }, method = RequestMethod.GET)
	public String update(@PathVariable Integer id, ModelMap model) {
		Customer customer = customerService.find(id);
		model.addAttribute("customer", customer);
		model.addAttribute("isNew", false);
		
		//Getting list of orders
		List<Orders> orders = ordersService.listByCustomer(customer, null, null);
		model.addAttribute("orders", orders);
		
		return "customer/form";
	}
	
	/**
	 * Update form from Order delete
	 */
	@RequestMapping(value = { "/update/{id}/{status}" }, method = RequestMethod.GET)
	public String update(@PathVariable Integer id, @PathVariable String status, ModelMap model) {
		if(status.equals("OK")) {
			model.addAttribute("success", "Update order successful");	
		} else {
			model.addAttribute("error", "Update order failed");
		}		
		
		return update(id, model);
	}
	
	/**
	 * Save update
	 */
	@RequestMapping(value = { "/update/{id}" }, method = RequestMethod.POST)
	public String update(
			@Valid Customer customer, BindingResult result, 
			ModelMap model, @PathVariable Integer id) {
		if (result.hasErrors()) {
			model.addAttribute("error", "Customer " + customer.getEmail() + " updated failed");
			model.addAttribute("customer", customer);
			model.addAttribute("isNew", false);	
			
			return "customer/form";
		}

		//Update Customer
		customerService.save(customer);
		
		model.addAttribute("success", "Customer " + customer.getEmail() + " updated successfully");
		model.addAttribute("customer", customer);
		model.addAttribute("isNew", false);
		
		return "customer/form";
	}
	
	/**
	 * Delete by ID
	 */
	@RequestMapping(value = { "/delete/{id}" }, method = RequestMethod.GET)
	public String delete(ModelMap model, @PathVariable Integer id) {
		Customer customer = customerService.find(id);
		if(customer!=null) {
			customerService.deleteByID(id);
			
			return list(model, "Customer ID : " + id + " deleted successfully");
		} else {
			return list(model, "Cannot find Customer ID : " + id + ", fail to deleted");
		}
	}
	
	/**
	 * CustomerController controller will return this model attribute to the views
	 */
	@ModelAttribute("genders")
	public Map<String, String> genderList() {		
		Map<String, String> genders = new LinkedHashMap<String, String>();
		genders.put("", "");
		genders.put("M", "Male");
		genders.put("F", "Female");
		
		return genders;
	}
}