package com.itblogs.controller;

import java.beans.PropertyEditorSupport;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import com.itblogs.service.*;
import com.itblogs.model.*;

/**
 * @author      Kelvin Hui
 * @see       	it-blogs.com
 * @since     	01-Jan-2016
 */

@Controller
@RequestMapping("/order")
public class OrdersController {

	private static final Logger logger = Logger.getLogger(OrdersController.class);
	
	@Autowired
	CustomerService customerService;
	
	@Autowired
	OrdersService ordersService;
	
	@Autowired
	ProductService productService;
	
    @Autowired
    MessageSource messageSource;

    
	/**
	 * Listing all existing records
	 */    
    @RequestMapping(value = { "/", "/list" }, method = RequestMethod.GET)
    public String list(ModelMap model, String msg) {
    	List<Orders> orders = ordersService.list(null,null);
    	model.addAttribute("orders", orders);
    	model.addAttribute("msg", msg);
    	
        return "order/list";
    }
 
	/**
	 * Listing all existing records for specified customer
	 */    
    @RequestMapping(value = { "/", "/list/{customerId}" }, method = RequestMethod.GET)
    public String list(@PathVariable Integer customerId, ModelMap model, String msg) {
    	List<Orders> orders = ordersService.list(null,null);
    	model.addAttribute("orders", orders);
    	model.addAttribute("msg", msg);
    	
        return "order/list";
    }
    
    /**
	 * New form
	 */
	@RequestMapping(value = { "/new" }, method = RequestMethod.GET)
	public String newForm(ModelMap model) {
		model = processNewForm(null, model);
		return "order/form";
	}

    /**
	 * New form from Customer update form
	 */
	@RequestMapping(value = { "/new/{customerId}" }, method = RequestMethod.GET)
	public String newForm(@PathVariable Integer customerId, ModelMap model) {
		model = processNewForm(customerId, model);		
		return "order/form";
	}
	
	private ModelMap processNewForm(Integer customerId, ModelMap model) {
		Orders order = new Orders();		
		if(customerId!=null) {
			Customer customer = customerService.find(customerId);
			order.setCustomer(customer);
		}
		model.addAttribute("order", order);		
		model.addAttribute("isNew", true);
		model.addAttribute("customerId", customerId);
		model = customersList(model);
		model = productsList(model);
		
		return model;
	}
	
	/**
	 * Save new
	 */
	@RequestMapping(value = { "/new" }, method = RequestMethod.POST)
	public String save(
			@Valid Orders order, BindingResult result, 
			ModelMap model) {
		processSave(order, result, model);
		
		return "order/form";
	}
	
	/**
	 * Save new from Customer update form
	 */
	@RequestMapping(value = { "/new/{customerId}" }, method = RequestMethod.POST)
	public String save(
			@PathVariable Integer customerId, 
			@Valid Orders order, BindingResult result, 
			ModelMap model) {
		processSave(order, result, model);
		
		if(customerId!=null) {
			return "redirect:/customer/update/"+customerId;	
		} else {
			return "order/form";	
		}		
	}
	
	private ModelMap processSave(
			Orders order, BindingResult result, 
			ModelMap model) {
		model = customersList(model);
		model = productsList(model);
		
		if (result.hasErrors()) {
			model.addAttribute("error", "Order " + order.getId() + " created failed");
			model.addAttribute("order", order);
			model.addAttribute("isNew", true);
			
System.out.println("result="+result);
			
			return model;
		}

		//Save Order		
		ordersService.save(order);

		model.addAttribute("success", "Order " + order.getId() + " created successfully");
		model.addAttribute("order", order);
		model.addAttribute("isNew", false);
		
		return model;
	}
		
	/**
	 * Update form
	 */
	@RequestMapping(value = { "/update/{id}" }, method = RequestMethod.GET)
	public String update(@PathVariable Integer id, ModelMap model) {
		
		logger.info("GET /update/{id}" + id);
		
		model = processUpdateForm(id, null, model);

		return "order/form";
	}
	
	/**
	 * Update form from Customer update form
	 */
	@RequestMapping(value = { "/update/{id}/{customerId}" }, method = RequestMethod.GET)
	public String update(
			@PathVariable Integer id, 
			@PathVariable Integer customerId, 
			ModelMap model) {
		
		logger.info("GET /update/{id}/{customerId}" + id + ":" + customerId);
		
		model = processUpdateForm(id, customerId, model);
		
		return "order/form";
	}
	
	private ModelMap processUpdateForm(Integer id, Integer customerId, ModelMap model) {
		Orders order = ordersService.find(id);
		model.addAttribute("order", order);
		model.addAttribute("isNew", false);
		model = customersList(model);
		model = productsList(model);
		model.addAttribute("customerId", customerId);
		
		return model;
	}
	
	/**
	 * Save update
	 */
	@RequestMapping(value = { "/update/{id}" }, method = RequestMethod.POST)
	public String update(
			@Valid Orders order, BindingResult result, 
			ModelMap model, @PathVariable Integer id) {
		
		logger.info("POST /update/{id}" + id);
		
		model = customersList(model);
		model = productsList(model);
		processUpdate(order, result, model, id);
		
		return "order/form";
	}
	
	/**
	 * Save update from Customer update form
	 */
	@RequestMapping(value = { "/update/{id}/{customerId}" }, method = RequestMethod.POST)
	public String update(
			@Valid Orders order, BindingResult result, 
			ModelMap model, @PathVariable Integer id, @PathVariable Integer customerId) {
		
		logger.info("POST /update/{id}" + id + ":" + customerId);
		
		processUpdate(order, result, model, id);
		
		if (result.hasErrors()) {
			return "order/form";	
		} else {
			return "redirect:/customer/update/"+customerId+"/OK";
		}
	}	
	
	private ModelMap processUpdate(
			@Valid Orders order, BindingResult result, 
			ModelMap model, Integer id) {
		if (result.hasErrors()) {
			model.addAttribute("error", "Order " + order.getId() + " updated failed");
			model.addAttribute("order", order);
			model.addAttribute("isNew", false);	
			
			return model;
		}

		//Update Order
		ordersService.save(order);
		
		model.addAttribute("success", "Order " + order.getId() + " updated successfully");
		model.addAttribute("order", order);
		model.addAttribute("isNew", false);
		
		return model;
	}
	
	/**
	 * Delete by ID
	 */
	@RequestMapping(value = { "/delete/{id}" }, method = RequestMethod.GET)
	public String delete(ModelMap model, @PathVariable Integer id) {
		
		logger.info("/delete/{id}" + id);
		
		Orders order = ordersService.find(id);
		if(order!=null) {
			ordersService.deleteByID(id);
			
			return list(model, "Order ID : " + id + " deleted successfully");
		} else {
			return list(model, "Cannot find Order ID : " + id + ", fail to deleted");
		}
	}
	
	/**
	 * Delete by ID from Customer update form
	 */
	@RequestMapping(value = { "/delete/{id}/{customerId}" }, method = RequestMethod.GET)
	public String delete(ModelMap model, @PathVariable Integer id, @PathVariable Integer customerId) {
		
		logger.info("/delete/{id}" + id + ":" + customerId);
		
		Orders order = ordersService.find(id);
		if(order!=null) {
			ordersService.deleteByID(id);
			
			return "redirect:/customer/update/"+customerId+"/OK";
		} else {
			return "redirect:/customer/update/"+customerId+"/FAIL";
		}
	}
	
	private ModelMap customersList(ModelMap model) {		
		List<Customer> customerList = customerService.list(null,null);
		model.addAttribute("customers", customerList);
		
		return model;
	}
	private ModelMap productsList(ModelMap model) {		
		List<Product> productsList = productService.list(null,null);
		model.addAttribute("productsList", productsList);
		
		return model;
	}
	
	@InitBinder
	protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
		binder.registerCustomEditor(Product.class, "products", new PropertyEditorSupport() {	
			@Override
			public void setAsText(String IdTxt) {
				Product product = productService.find(Integer.parseInt(IdTxt));
				setValue(product);
			}
		});
	}
}