package com.itblogs.controller;

import java.util.List;
import javax.validation.Valid;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import com.itblogs.service.*;
import com.itblogs.model.*;

/**
 * @author      Kelvin Hui
 * @see       	it-blogs.com
 * @since     	01-Jan-2016
 */

@Controller
@RequestMapping("/product")
public class ProductController {

	private static final Logger logger = Logger.getLogger(ProductController.class);
	
	@Autowired
	ProductService productService;
	
    @Autowired
    MessageSource messageSource;

    
	/**
	 * Listing all existing records
	 */    
    @RequestMapping(value = { "/", "/list" }, method = RequestMethod.GET)
    public String list(ModelMap model, String msg) {
    	List<Product> products = productService.list(null,null);
    	model.addAttribute("products", products);
    	model.addAttribute("msg", msg);
    	
        return "product/list";
    }
 
    /**
	 * New form
	 */
	@RequestMapping(value = { "/new" }, method = RequestMethod.GET)
	public String newForm(ModelMap model) {
		Product product = new Product();
		model.addAttribute("product", product);		
		model.addAttribute("isNew", true);		
		
		return "product/form";
	}
 
	/**
	 * Save new
	 */
	@RequestMapping(value = { "/new" }, method = RequestMethod.POST)
	public String save(
			@Valid Product product, BindingResult result, 
			ModelMap model) {
		if (result.hasErrors()) {
			model.addAttribute("error", "Product " + product.getName() + " created failed");
			model.addAttribute("product", product);
			model.addAttribute("isNew", true);
			
			return "product/form";
		}

		//Save Product		
		productService.save(product);

		model.addAttribute("success", "Product " + product.getName() + " created successfully");
		model.addAttribute("product", product);
		model.addAttribute("isNew", false);
		
		return "product/form";
	}
	
	/**
	 * Update form
	 */
	@RequestMapping(value = { "/update/{id}" }, method = RequestMethod.GET)
	public String update(@PathVariable Integer id, ModelMap model) {
		Product product = productService.find(id);
		model.addAttribute("product", product);
		model.addAttribute("isNew", false);
		
		return "product/form";
	}
	
	/**
	 * Save update
	 */
	@RequestMapping(value = { "/update/{id}" }, method = RequestMethod.POST)
	public String update(
			@Valid Product product, BindingResult result, 
			ModelMap model, @PathVariable Integer id) {
		if (result.hasErrors()) {
			model.addAttribute("error", "Product " + product.getName() + " updated failed");
			model.addAttribute("product", product);
			model.addAttribute("isNew", false);	
			
			return "product/form";
		}

		//Update Product
		productService.save(product);
		
		model.addAttribute("success", "Product " + product.getName() + " updated successfully");
		model.addAttribute("product", product);
		model.addAttribute("isNew", false);
		
		return "product/form";
	}
	
	/**
	 * Delete by ID
	 */
	@RequestMapping(value = { "/delete/{id}" }, method = RequestMethod.GET)
	public String delete(ModelMap model, @PathVariable Integer id) {
		Product product = productService.find(id);
		if(product!=null) {
			productService.deleteByID(id);
			
			return list(model, "Product ID : " + id + " deleted successfully");
		} else {
			return list(model, "Cannot find Product ID : " + id + ", fail to deleted");
		}
	}
	
}