package com.itblogs.controller;

import java.util.List;
import javax.validation.Valid;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import com.itblogs.service.*;
import com.itblogs.model.*;

/**
 * @author      Kelvin Hui
 * @see       	it-blogs.com
 * @since     	01-Jan-2016
 */

@Controller
@RequestMapping("/shop")
public class ShopController {

	private static final Logger logger = Logger.getLogger(ShopController.class);
	
	@Autowired
	ShopService shopService;
	
	@Autowired
	AddressService addressService;
	
    @Autowired
    MessageSource messageSource;

    
	/**
	 * Listing all existing records
	 */    
    @RequestMapping(value = { "/", "/list" }, method = RequestMethod.GET)
    public String list(ModelMap model, String msg) {
    	List<Shop> shops = shopService.list(null,null);
    	model.addAttribute("shops", shops);
    	model.addAttribute("msg", msg);
    	
        return "shop/list";
    }
 
    /**
	 * New form
	 */
	@RequestMapping(value = { "/new" }, method = RequestMethod.GET)
	public String newForm(ModelMap model) {
		Shop shop = new Shop();
		model.addAttribute("shop", shop);		
		model.addAttribute("isNew", true);		
		
		return "shop/form";
	}
 
	/**
	 * Save new
	 */
	@RequestMapping(value = { "/new" }, method = RequestMethod.POST)
	public String save(
			@Valid Shop shop, BindingResult result, 
			ModelMap model) {
		if (result.hasErrors()) {
			model.addAttribute("error", "Shop " + shop.getName() + " created failed");
			model.addAttribute("shop", shop);
			model.addAttribute("isNew", true);	
			
			return "shop/form";
		}

		//Save Address
		Address address = new Address(shop.getAddress().getBuilding(), shop.getAddress().getStreet(), shop.getAddress().getCity(), shop.getAddress().getCountry());
		addressService.save(address);
		
		shop.setAddress(address);
		
		//Save Shop		
		shopService.save(shop);

		model.addAttribute("success", "Shop " + shop.getName() + " created successfully");
		model.addAttribute("shop", shop);
		model.addAttribute("isNew", false);
		
		return "shop/form";
	}
	
	/**
	 * Update form
	 */
	@RequestMapping(value = { "/update/{id}" }, method = RequestMethod.GET)
	public String update(@PathVariable Integer id, ModelMap model) {
		Shop shop = shopService.find(id);
		model.addAttribute("shop", shop);
		model.addAttribute("isNew", false);
		
		return "shop/form";
	}
	
	/**
	 * Save update
	 */
	@RequestMapping(value = { "/update/{id}" }, method = RequestMethod.POST)
	public String update(
			@Valid Shop shop, BindingResult result, 
			ModelMap model, @PathVariable Integer id) {
		if (result.hasErrors()) {
			model.addAttribute("error", "Shop " + shop.getName() + " updated failed");
			model.addAttribute("shop", shop);
			model.addAttribute("isNew", false);	
			
			return "shop/form";
		}

		//Update address
		addressService.save( shop.getAddress() );
		
		//Update shop
		shopService.save(shop);
		
		model.addAttribute("success", "Shop " + shop.getName() + " updated successfully");
		model.addAttribute("shop", shop);
		model.addAttribute("isNew", false);
		
		return "shop/form";
	}
	
	/**
	 * Delete by ID
	 */
	@RequestMapping(value = { "/delete/{id}" }, method = RequestMethod.GET)
	public String delete(ModelMap model, @PathVariable Integer id) {
		Shop shop = shopService.find(id);
		if(shop!=null) {
			shopService.deleteByID(id);
			
			return list(model, "Shop ID : " + id + " deleted successfully");
		} else {
			return list(model, "Cannot find Shop ID : " + id + ", fail to deleted");
		}
	}
}