package com.itblogs.dao.impl;

import java.sql.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Hibernate;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.itblogs.dao.*;
import com.itblogs.model.*;

/**
 * @author      Kelvin Hui
 * @see       	it-blogs.com
 * @since     	01-Jan-2016
 */

@Repository("ordersDao")
public class OrdersDaoImpl extends GenericDao<Orders> implements OrdersDao {
	
	public Orders find(Integer id) {
		Orders orders = findByID(id); 
		if(orders!=null){
			Hibernate.initialize(orders.getProducts());
		}		
		return orders;
	}
	
	public List<Orders> list(String orderBy, Boolean isAse) {
		return listAll(orderBy, isAse);
	}

	public List<Orders> listByCustomer(Customer customer, String orderBy, Boolean isAse) {
		Criteria criteria = createEntityCriteria();
		if(customer!=null) criteria.add(Restrictions.eq("customer", customer));
		if(orderBy!=null)
			if(isAse) { criteria.addOrder(Order.asc(orderBy)); } else { criteria.addOrder(Order.desc(orderBy)); }
        return (List<Orders>)criteria.list();
	}
	
	public List<Orders> listByDate(Date orderDate, Boolean status, String orderBy, Boolean isAse) {
		Criteria criteria = createEntityCriteria();
		if(orderDate!=null) criteria.add(Restrictions.eq("orderDate", orderDate));
		if(status!=null) criteria.add(Restrictions.eq("status", status.booleanValue()));
		if(orderBy!=null)
			if(isAse) { criteria.addOrder(Order.asc(orderBy)); } else { criteria.addOrder(Order.desc(orderBy)); }
        return (List<Orders>)criteria.list();
	}
	
	public long count() {
		return countAll();
	}
	
	public void save(Orders orders) {
		saveOrUpdate(orders);
	}
	
	public void delete(Orders orders) {
		deleteRecord(orders);
	}
	
	public void deleteByID(Integer id) {
		deleteRecordByID(id);
	}
	
	public boolean contains(Integer id) {
		return containsRecord(id);
	}
		
}
