package com.itblogs.dao.impl;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.itblogs.dao.*;
import com.itblogs.model.Product;

/**
 * @author      Kelvin Hui
 * @see       	it-blogs.com
 * @since     	01-Jan-2016
 */

@Repository("productDao")
public class ProductDaoImpl extends GenericDao<Product> implements ProductDao {

	public Product find(Integer id) {
        return findByID(id);
	}
	
	public Product findByName(String name) {
		Criteria criteria = createEntityCriteria();
		criteria.add(Restrictions.eq("name", name));
        return (Product)criteria.uniqueResult();
	}
	
	public Product findByStatus(Boolean status) {
		Criteria criteria = createEntityCriteria();
		if(status!=null) criteria.add(Restrictions.eq("status", status));
        return (Product)criteria.uniqueResult();
	}
	
	public List<Product> list(String orderBy, Boolean isAse) {
		return listAll(orderBy, isAse);
	}
	
	public List<Product> list(Boolean status, String orderBy, Boolean isAse) {		
		Criteria criteria = createEntityCriteria();
		if(status!=null) criteria.add(Restrictions.eq("status", status));
        return (List<Product>)criteria.list();
	}
	
	public long count() {
		return countAll();
	}
	
	public void save(Product product) {
		saveOrUpdate(product);
	}
	
	public void delete(Product product) {
		deleteRecord(product);
	}
	
	public void deleteByID(Integer id) {
		deleteRecordByID(id);
	}
	
	public void deleteByName(String name) {
		Product product = findByName(name);
		if(product!=null) 
			deleteRecord(product);
	}
	
	public boolean contains(Integer id) {
		return containsRecord(id);
	}
}
