package com.itblogs.service.impl;

import java.util.List;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.context.annotation.ScopedProxyMode;

import com.itblogs.model.Product;
import com.itblogs.dao.ProductDao;
import com.itblogs.service.ProductService;

/**
 * @author      Kelvin Hui
 * @see       	it-blogs.com
 * @since     	01-Jan-2016
 */

@Service("productService")
@Transactional
@Scope(value="session", proxyMode=ScopedProxyMode.TARGET_CLASS)
public class ProductServiceImpl implements ProductService {

	@Autowired
	ProductDao dao;
	
	public Product find(Integer id) {
		return dao.find(id);
	}
	
	public Product findByName(String name) {
		if(name==null) {
			return null;
		} else {
			return dao.findByName(name);	
		}
	}
	
	public Product findByStatus(Boolean status) {
		if(status==null) {
			return null;
		} else {
			return dao.findByStatus(status);	
		}
	}
	
	public List<Product> list(String orderBy, Boolean isAse) {		
		return dao.list(orderBy, isAse);
	}
	
	public List<Product> list(Boolean status, String orderBy, Boolean isAse) {
		if(status==null) {
			return null;
		} else {
			return dao.list(status, orderBy, isAse);	
		}
	}

	public long count() {
		return dao.count();
	}

	public void save(Product product) {
		if(product!=null) dao.save(product);
	}

	public void delete(Product product) {
		if(product!=null) dao.delete(product);
	}

	public void deleteByID(Integer id) {
		dao.deleteByID(id);
	}

	public void deleteByName(String name) {
		if(name!=null) dao.deleteByName(name);	
	}
	
	public boolean contains(Integer id) {
		return dao.contains(id);
	}
	
}
